describe('PreferenceManager', function () {

	var zhPromise = new ZHPromise(Q);
	var thenable = function(data) {
		return {
			then : function(fn){
				var dataReturned;
				if(fn){
					dataReturned = fn(data);
				}
				return thenable(dataReturned || data);
			}
		};
	};

	var localStorageEngine;
	var requestDataSourceEngine;
	var synchronizableStorageEngineFactory;
	var synchronizableStorageEngine;
	var preferenceManager;

	var defaultContainerPreference = {
		"name": "login",
		"showFooter": false,
		"widgets": [
			{
				"name": "loginWidget",
				"fields": [{
					"label": "Default label"
				}]
			}
		]
	};

	var buildMock = function() {
		var mock = {
			whenSuccess: function() {},
			whenError: function() {}
		};
		spyOn(mock, 'whenSuccess').and.callThrough();
		spyOn(mock, 'whenError').and.callThrough();
		return mock;
	};

	beforeEach(function(){
		requestDataSourceEngine = {
			save: function(){
				return thenable();
			}
		};
		localStorageEngine = {
			save: function(){
				return thenable();
			},
			getDataSource: function(){
				return thenable([defaultContainerPreference]);
			},
			sync: function(){
				return thenable([defaultContainerPreference]);
			}
		};
		var repoTest;
		synchronizableStorageEngineFactory = new SynchronizableStorageEngineFactory(zhPromise);

		var zhUtil = {
			registerDataSource : function(name){
				repoTest = name;
		}};

		var repositoryFactory = {
			factory: function(){
				return {
					find: function(){
						return zhPromise.when([defaultContainerPreference]);
					},
					save: function(){
						return {
							then: function(callback){
								return	callback();
							}
						};
					}
				};
			}
		};

	preferenceManager = new PreferenceManager(synchronizableStorageEngineFactory, {}, localStorageEngine, requestDataSourceEngine, new QueryBuilder(), { i18n: function(string){return string;}}, zhPromise, zhUtil, repositoryFactory);

});

	it('The PreferenceManager must be instanteate with the default properties', function(){
		var mock = buildMock();
		preferenceManager.storePreference('login', {}).then(function(){
			mock.whenSuccess();
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
			//expect(requestDataSourceEngine.save).not.toHaveBeenCalled();
		},function(){
			mock.whenError();
		});

	});

	it('The PreferenceManager should change its storageStrategy properties', function(){
		preferenceManager.modifyStorageStrategy(localStorageEngine, requestDataSourceEngine, {}, false, true);
		var mock = buildMock();
		preferenceManager.storePreference('login', {}).then(function(){
			mock.whenSuccess();
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
		}, function(){
			mock.whenError();
		});
	});

	it('Method "storePreference": must be save the preference data', function(){
		var containerPreference = {
			'name': 'login',
			'widgets': [
				{
					'name': 'loginWidget',
					'fields': [{
						'label': 'Modified label'
					}]
				}
			]
		};

		var mock = buildMock();
		preferenceManager.storePreference('login', containerPreference).then(mock.whenSuccess, mock.whenError);

		expect(mock.whenSuccess).toHaveBeenCalled();
		expect(mock.whenError).not.toHaveBeenCalled();
	});

	it('Method "storePreference": should return a reject promise if has invalid arguments', function(done){
		var containerPreference = {
			'name': 'login',
			'widgets': [
				{
					'name': 'loginWidget',
					'fields': [{
						'label': 'Modified label'
					}]
				}
			]
		};

		preferenceManager.storePreference('login').then(null, function(err){
			expect(err.err).toEqual('Invalid arguments');
			preferenceManager.storePreference(null, containerPreference).then(null, function(err){
				expect(err.err).toEqual('Invalid arguments');
				done();
			});
		});

	});

	it('Method "load": should return your custom preferences of a container', function(){
		var mock = buildMock();
		preferenceManager.load('login').then(function(container){
			mock.whenSuccess();
			expect(container.name).toEqual('login');
			expect(container.showFooter).toEqual(false);
			expect(container.widgets[0].name).toEqual('loginWidget');
			expect(container.widgets[0].fields[0].label).toEqual('Default label');
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
		}, mock.whenError);

	});

	it('Method "sync": should synchronize your storageStrategy', function(){
		var mock = buildMock();
		preferenceManager.sync().then(function(container){
			mock.whenSuccess();
			expect(container[0].name).toEqual('login');
			expect(container[0].showFooter).toEqual(false);
			expect(container[0].widgets[0].name).toEqual('loginWidget');
			expect(container[0].widgets[0].fields[0].label).toEqual('Default label');
			expect(mock.whenSuccess).toHaveBeenCalled();
			expect(mock.whenError).not.toHaveBeenCalled();
		}, mock.whenError);

	});

});